import numpy as np
import settings
import random
import lmdb
import sys
import matplotlib.pyplot as plt
sys.path.insert(0, settings.caffe_root + 'python')
import caffe

class Visualizator:

    def visualize(self, directory):

        train_images_env = lmdb.open(directory + 'train_rush_images', readonly=True)
        train_labels_env = lmdb.open(directory + 'train_rush_labels', readonly=True)
        val_images_env = lmdb.open(directory + 'val_rush_images', readonly=True)
        val_labels_env = lmdb.open(directory + 'val_rush_labels', readonly=True)

        images = []
        targets = []

        with train_images_env.begin() as txn_train_images:
            with train_labels_env.begin() as txn_train_labels:

                images_cursor = txn_train_images.cursor()
                targets_cursor = txn_train_labels.cursor()

                length = txn_train_images.stat()['entries']
                print(length)

                id_images = []
                for i in range(0,3):
                    id_images.append(random.randint(0, 501-1))

                for id_img in id_images:
                    str_id = '{:08}'.format(id_img)

                    value_images = images_cursor.get(str_id.encode('ascii'))
                    value_targets = targets_cursor.get(str_id.encode('ascii'))

                    datum = caffe.proto.caffe_pb2.Datum()

                    datum.ParseFromString(value_images)
                    flat_img = np.fromstring(datum.data, dtype=np.uint8)
                    img = flat_img.reshape(datum.channels, datum.height, datum.width)
                    img = img.transpose((1, 2, 0))
                    images.append(img)

                    datum.ParseFromString(value_targets)
                    target = caffe.io.datum_to_array(datum)

                    targets.append(target)

        with val_images_env.begin() as txn_val_images:
            with val_labels_env.begin() as txn_val_labels:

                images_cursor = txn_val_images.cursor()
                targets_cursor = txn_val_labels.cursor()

                length = txn_val_images.stat()['entries']
                print(length)

                id_images = []
                for i in range(0, 3):
                    id_images.append(random.randint(0, 501 - 1))

                for id_img in id_images:
                    str_id = '{:08}'.format(id_img)

                    value_images = images_cursor.get(str_id.encode('ascii'))
                    value_targets = targets_cursor.get(str_id.encode('ascii'))

                    datum = caffe.proto.caffe_pb2.Datum()

                    datum.ParseFromString(value_images)
                    flat_img = np.fromstring(datum.data, dtype=np.uint8)
                    img = flat_img.reshape(datum.channels, datum.height, datum.width)
                    img = img.transpose((1, 2, 0))
                    images.append(img)

                    datum.ParseFromString(value_targets)
                    target = caffe.io.datum_to_array(datum)

                    targets.append(target)

        print(len(images))

        labels = []
        with open(directory + 'train_stats.txt', 'r') as file:
            for line in file:
                if 'Number of images for' in line:
                    label = line.split(' ')[-3]
                    labels.append(label)

        plt.figure(figsize=(15,6))
        plt.suptitle(' - '.join(labels))
        cpt = 1
        for image, target in zip(images, targets):

            plt.subplot(2,3,cpt)
            plt.imshow(image)
            plt.title(target)
            cpt += 1

        plt.show()









import MySQLdb as mdb
import numpy as np
import matplotlib.pyplot as plt
import sys

caffe_root = '/home/leo/caffe/'  # this file should be run from {caffe_root}/examples (otherwise change this line)
sys.path.insert(0, caffe_root + 'python')
import caffe
import glob
import os


def init_network(model_name):
    # set display defaults
    plt.rcParams['figure.figsize'] = (10, 10)  # large images
    plt.rcParams['image.interpolation'] = 'nearest'  # don't interpolate: show square pixels
    plt.rcParams['image.cmap'] = 'gray'  # use grayscale output rather than a (potentially misleading) color heatmap

    caffe.set_mode_gpu()

    model_def = caffe_root + 'resources/' + str(model_name) + '/deploy.prototxt'
    model_weights = caffe_root + 'resources/' + str(model_name) + '/' + str(model_name) + '.caffemodel'

    net = caffe.Net(model_def,  # defines the structure of the model
                    model_weights,  # contains the trained weights
                    caffe.TEST)  # use test mode (e.g., don't perform dropout)

    # load the mean ImageNet image (as distributed with Caffe) for subtraction
    # mu = np.load(caffe_root + 'python/caffe/imagenet/ilsvrc_2012_mean.npy')
    # mu = mu.mean(1).mean(1)  # average over pixels to obtain the mean (BGR) pixel values
    # print 'mean-subtracted values:', zip('BGR', mu)

    # create transformer for the input called 'data'
    transformer = caffe.io.Transformer({'data': net.blobs['data'].data.shape})

    transformer.set_transpose('data', (2, 0, 1))  # move image channels to outermost dimension
    # transformer.set_mean('data', mu)            # subtract the dataset-mean value in each channel
    transformer.set_raw_scale('data', 255)  # rescale from [0, 1] to [0, 255]
    transformer.set_channel_swap('data', (2, 1, 0))  # swap channels from RGB to BGR

    # set the size of the input (we can skip this if we're happy
    #  with the default; we can also change it later, e.g., for different batch sizes)

    if model_name.endswith('caffenet'):
        net.blobs['data'].reshape(1,  # batch size
                                  3,  # 3-channel (BGR) images
                                  227, 227)  # image size is 227x227
    elif model_name.endswith('network'):
        net.blobs['data'].reshape(50,  # batch size
                                  3,  # 3-channel (BGR) images
                                  224, 224)  # image size is 227x227

    return net, transformer


def net_features(model_name):
    con = mdb.connect('localhost', 'leo', 'castor69', 'webcastor', charset='utf8', use_unicode=True)
    cur = con.cursor()

    net, transformer = init_network(model_name)

    imagenames = glob.glob('../../../images/*')

    for imagename in imagenames:
        media_ref = imagename.split('/')[-1].split('.')[0].split('_')[0]
        name = imagename.split('/')[-1].split('.')[0]

        # cur.execute('SELECT media FROM features WHERE imagename = %s', \
        # 			 (name,) )

        # media = cur.fetchone()

        # if media is None:

        image = caffe.io.load_image(imagename)

        transformed_image = transformer.preprocess('data', image)

        # copy the image data into the memory allocated for the net
        net.blobs['data'].data[...] = transformed_image

        ### perform classification
        output = net.forward()
        last_feats = net.blobs['loss3/classifier'].data

        features = last_feats[0, :]

        # print(np.array(features).shape)

        cur.execute('UPDATE features SET vgg_features = %s WHERE imagename = %s', (str(features), name))
        con.commit()


def more_net_features(model_name):
    con = mdb.connect('localhost', 'leo', 'castor69', 'webcastor', charset='utf8', use_unicode=True)
    cur = con.cursor()

    net, transformer = init_network(model_name)

    imagenames = []
    for root, dirnames, filenames in os.walk('../../more_images'):

        for dirname in dirnames:
            names = glob.glob('../../more_images/' + dirname + '/*')
            imagenames = imagenames + names

    for imagename in imagenames:

        media_ref = imagename.split('/')[-1].split('.')[0].split('_')[0]
        name = imagename.split('/')[-1].split('.')[0]

        cur.execute('SELECT media FROM features WHERE imagename = %s', \
                    (name,))

        media = cur.fetchone()

        if media is None:
            image = caffe.io.load_image(imagename)
            transformed_image = transformer.preprocess('data', image)

            # copy the image data into the memory allocated for the net
            net.blobs['data'].data[...] = transformed_image

            ### perform classification
            output = net.forward()

            last_feats = net.blobs['fc8'].data

            features = last_feats[0, :]

            cur.execute('INSERT INTO features (caffe_features, media, imagename, more_images) VALUES (%s, %s, %s, %s)', \
                        (str(features), media_ref, name, 1))
            con.commit()

# model_name = sys.argv[1]
# print(model_name)
# net_features(model_name)
# more_net_features()



class DeepVideoBaseException(Exception):
    pass

class MissingLabel(DeepVideoBaseException):

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


import unicodedata
import shutil
import os
import glob

def remove_accents(input_str):
    nkfd_form = unicodedata.normalize('NFKD', unicode(input_str))
    return u"".join([c for c in nkfd_form if not unicodedata.combining(c)])

def transfer_files():

    working_directory = '/home/leo/Desktop/graph_test/'

    dst = os.path.join(os.path.dirname(__file__), '../resources/tags/')
    shutil.copyfile(working_directory + 'network/deploy.prototxt', dst + 'deploy.prototxt')

    dst = os.path.join(os.path.dirname(__file__), '../resources/tags/onto.graphml')
    shutil.copyfile(working_directory + 'onto.graphml', dst)

    if not os.path.exists(os.path.join(os.path.dirname(__file__), '../resources/tags/weights/')):
        os.makedirs(os.path.join(os.path.dirname(__file__), '../resources/tags/weights/'))

    if not os.path.exists(os.path.join(os.path.dirname(__file__), '../resources/tags/thresholds/')):
        os.makedirs(os.path.join(os.path.dirname(__file__), '../resources/tags/thresholds/'))

    for dir in os.listdir(working_directory):

        if os.path.isdir(os.path.join(working_directory, dir)):

            if os.path.exists(os.path.join(working_directory, dir + '/training')) and os.path.exists(os.path.join(working_directory, dir + '/eval')):

                dst = os.path.join(os.path.dirname(__file__), '../resources/tags/weights/' + dir + '.caffemodel')
                src = glob.glob(os.path.join(working_directory, dir + '/training/*.caffemodel'))
                shutil.copyfile(src[0], dst)

                dst = os.path.join(os.path.dirname(__file__), '../resources/tags/thresholds/' + dir + '.pkl')
                src = os.path.join(working_directory, dir + '/eval/thresholds.pkl')
                shutil.copyfile(src, dst)

if __name__ == '__main__':

    transfer_files()


# -*- coding: utf-8 -*-
import numpy as np
import cv2
import sys
from scipy.spatial.distance import euclidean
import math
from matplotlib import pyplot as plt
from matplotlib.pyplot import cm
import warnings
warnings.filterwarnings('error')
import logging
import itertools

CHANGED_POINTS_FRACTION = 0.5
CHANGED_ORIENTATION_FRACTION = 0.6
CHANGE_THRESHOLD = 0.9
SEQ_THRESHOLD = 0.1

def cp_directions(new, old):
    """
    Compute the direction between two points

    Args:
        @type new: Tuple of int
        @param new: New keypoints

        @type new: Tuple of int
        @param new: Old keypoints

    Return:
        @type direction: float
        @param direction: Direction between the old and new points

        @type norm: float
        @param norm: Norm between the old and new points

        @type orientation: float
        @param orientation: Orientation between the old and new points
    """

    dist = [old[0] - new[0], old[1] - new[1]]
    norm = math.sqrt(dist[0] ** 2 + dist[1] ** 2)
    denom = new[0] - old[0]
    if denom == 0:
        denom = 0.000001
    orientation = math.degrees(math.atan((new[1] - old[1])/denom))
    if norm == 0:
        norm = 0.0001
    direction = [dist[0] / norm, dist[1] / norm]

    return direction, norm, orientation

def change_detection(norms):
    """
    Detect if the camera is fixed or moving

    Args:
        @type norms: Array-like of int
        @param norms: All norms from all the keypoints pairs

    Return:
        @type static: boolean
        @param static: True if the camera is fixed, False otherwise
    """

    norms = np.array(norms)
    changed_points = np.sum(norms > CHANGE_THRESHOLD)

    if changed_points > len(norms) * CHANGED_POINTS_FRACTION:
        static = False
    else:
        static = True

    return static

def orientation_detection(orientations, directions):
    """
    Detect in which direction the camera is moving

    Args:
        @type orientations: Array-like of float
        @param orientations: All orientation from all the keypoints pairs

        @type directions: Array-like of float
        @param directions: All directions from all the keypoints pairs

    Return:
        @type orientation: String
        @param orientation: The direction in which the camera is moving
    """

    str_orientations = []
    for orientation, direction in zip(orientations, directions):

        if (45 > orientation > -45) and direction[0] < 0:
            str_orientations.append('pano droite gauche')
        elif (45 > orientation > -45) and direction[0] > 0:
            str_orientations.append('pano gauche droite')
        elif (135 > orientation > 45) and direction[1] > 0:
            str_orientations.append('pano haut bas')
        elif (-45 > orientation > -135) and direction[1] > 0:
            str_orientations.append('pano haut bas')
        elif (135 > orientation > 45) and direction[1] < 0:
            str_orientations.append('pano bas haut')
        elif (-45 > orientation > -135) and direction[1] < 0:
            str_orientations.append('pano bas haut')

    all_orientations, counts = np.unique(str_orientations, return_counts=True)

    orientation = None
    for orient, count in zip(all_orientations, counts):
        if count > len(orientations) * CHANGED_ORIENTATION_FRACTION:
            orientation = orient

    return orientation

def zoom_detection(good_new, good_old, frame_width_center, frame_height_center):
    """
    Detect in which direction the camera is moving

    Args:
        @type good_new: Array-like of tuple of int
        @param good_new: All keypoints from the new frame

        @type good_old: Array-like of tuple of int
        @param good_old:  All keypoints from the preceding frame

        @type frame_width_center: float
        @param frame_width_center: coordinate of the width center

        @type frame_height_center: float
        @param frame_height_center: coordinate of the height center

    Return:
        @type zoom: String
        @param zoom: The zoom being done
    """

    zooms = []
    for old, new in zip(good_old, good_new):

        old_dist = math.fabs(math.sqrt( (old[0] - frame_width_center )**2 + (old[1] - frame_height_center)**2 ))
        new_dist = math.fabs(math.sqrt( (new[0] - frame_width_center )**2 + (new[1] - frame_height_center)**2 ))

        if new_dist < old_dist :
            zoom = 'zoom arriere'
        else:
            zoom = 'zoom avant'

        # print(zoom)

        zooms.append(zoom)

    all_zooms, counts = np.unique(zooms, return_counts=True)

    zoom = all_zooms[np.argsort(counts)[::-1][0]]

    return zoom

def motion_detection(cap):
    """
    Process the frames of a video to identify the motion of the camera

    Args:
        @type cap: VideoCapture object
        @param cap: A VideoCapture object corresponding to a video file encapsulated by OpenCV

    Return:
        @type movement: String
        @param movement: The motion of the camera
    """

    # params useful for zoom detection
    frame_width = cap.shape[2]
    frame_height = cap.shape[1]
    frame_width_center = frame_width/2
    frame_height_center = frame_height/2

    # params for ShiTomasi corner detection
    feature_params = dict( maxCorners = 100,
                           qualityLevel = 0.3,
                           minDistance = 7,
                           blockSize = 7 )

    # Parameters for lucas kanade optical flow
    lk_params = dict( winSize  = (30,30),
                      maxLevel = 2,
                      criteria = (cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, 10, 0.03))

    # Take first frame and find corners in it
    # ret, old_frame = cap.read()
    # if ret == False:
    #     return -1
    old_frame = cap[0,:,:,:]

    # Find features to track in the first frame
    old_gray = cv2.cvtColor(old_frame, cv2.COLOR_BGR2GRAY)
    p0 = cv2.goodFeaturesToTrack(old_gray, mask = None, **feature_params)
    
    statics = []
    orientations = []
    zooms = []
    # total_directions = []
    # old_total_directions = []
    for i in range(1,cap.shape[0]):

        directions = []
        norms = []
        movements = []

        # ret,frame = cap.read()
        # if ret == False:
        #     break
        frame = cap[i,:,:,:]

        frame_gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)

        # calculate optical flow
        p1, st, err = cv2.calcOpticalFlowPyrLK(old_gray, frame_gray, p0, None, **lk_params)

        if st is None:
            break;

        # Select good points
        good_new = p1[st==1]
        good_old = p0[st==1]

        # Run through each pair of keypoints (for two concecutive frames) and compute useful metrics 
        for i, (new, old) in enumerate(zip(good_new,good_old)):

            direction, norm, movement = cp_directions(new, old)
            directions.append(direction)
            norms.append(norm)
            movements.append(movement)

        static = change_detection(norms)
        statics.append(static)
        if static == False:
            orientation = orientation_detection(movements, directions)
            orientations.append(orientation)
            if orientation == None:
                zoom = zoom_detection(good_new, good_old, frame_width_center, frame_height_center)
                zooms.append(zoom)

        # Now update the previous frame and previous points
        old_gray = frame_gray.copy()
        p0 = good_new.reshape(-1,1,2)

    # Compute the sequence max where frame is non static
    max_seq = 0
    for bit, group in itertools.groupby(statics):

        if bit == False:
            max = int(len(list(group)))
            if max > max_seq:
                max_seq = max
    
    # Determine the motion of the camera based on the metrics computed
    all_statics, counts = np.unique(statics, return_counts=True)

    if (max_seq < SEQ_THRESHOLD * len(statics)):
        movement = 'plan fixe'
    else:
        all_orientations, counts = np.unique(orientations, return_counts=True)

        movement = all_orientations[np.argsort(counts)[::-1][0]]

        if movement == None:
            all_zooms, counts = np.unique(zooms, return_counts=True)
            movement = all_zooms[np.argsort(counts)[::-1][0]]

    return movement

# videoname = sys.argv[1]
# cap = cv2.VideoCapture(videoname)
# movement = motion_detection(cap, False)
# print(movement)



class Reporter():

    def __init__(self):

        self.nb_images = 0
        self.corpus_images = None
        self.imagenet_images = {}

    def add_imagenet_informations(self, X, y):

        self.imagenet_images['total'] = len(X)


    class Factory:
        reporter = None

        def create(self):
            if not Reporter.Factory.interpreter:
                Reporter.Factory.interpreter = Reporter()

            return self.reporter


import numpy as np
import cv2
import sys
import matplotlib.pyplot as plt
from scipy.spatial.distance import cdist


def cluster_video(cap):
    # Initiate ORB detector
    orb = cv2.ORB(edgeThreshold=15, patchSize=31, nlevels=8, scaleFactor=1.2,
                         WTA_K=2, scoreType=cv2.ORB_HARRIS_SCORE, firstLevel=0, nfeatures=400)

    frames = []
    features = []
    for i in range(1, cap.shape[0]):

        frame = cap[i, :, :, :]
        # ret, frame = cap.read()
        # if ret == True:

        # find the keypoints with ORB
        kp = orb.detect(frame, None)

        # compute the descriptors with ORB
        kp, des = orb.compute(frame, kp)

        if len(kp) != 0:
            if des.shape[0] == 400:
                des = np.float32(des)
                features.append(des)
                frames.append(frame)

    features = np.array(features)
    frames = np.array(frames)

    # print(features.shape)

    if features.shape[0] > 50:

        # define criteria and apply kmeans()
        criteria = (cv2.TERM_CRITERIA_EPS + cv2.TERM_CRITERIA_MAX_ITER, 200, 0.1)
        flags = cv2.KMEANS_RANDOM_CENTERS
        # ret, labels, centers = cv2.kmeans(features, 2, [], criteria, 200, 0, flags)
        ret, labels, centers = cv2.kmeans(features, 2, criteria, 10, flags)

        features = np.reshape(features, (features.shape[0], features.shape[1] * features.shape[2]))

        best_frames = []

        Y = cdist(features, centers, 'euclidean')
        Y0 = Y[:, 0]
        Y1 = Y[:, 1]

        frame = frames[np.argsort(Y0)[0], :, :]
        best_frames.append(frame)

        frame = frames[np.argsort(Y1)[0], :, :]
        best_frames.append(frame)

    else:
        best_frames = []

    return best_frames

    # videoname = sys.argv[1]
    # cap = cv2.VideoCapture(videoname)
    # best_frames = cluster_video(cap)

    # for frame in best_frames:
    # 	plt.imshow(frame), plt.show()



from .interpreter import Interpreter
from .reporter import Reporter

class ObjectFactory:

    factories = {}

    def addFactory(id, factory):

        ObjectFactory.factories[id] = factory
    addFactory = staticmethod(addFactory)

    def createObject(id):

        if not ObjectFactory.factories.has_key(id):
            ObjectFactory.factories[id] = \
              eval(id + '.Factory()')
        return ObjectFactory.factories[id].create()
    createObject = staticmethod(createObject)



# import the necessary packages
from skimage.transform import pyramid_gaussian
import argparse
import cv2
import time
import imutils
import numpy as np
import matplotlib.pyplot as plt
import sys


def pyramid(image, scale=1.5, minSize=(30, 30)):
    # Resizing + Gaussian smoothing.
    for (i, resized) in enumerate(pyramid_gaussian(image, downscale=scale)):
        # if the image is too small, break from the loop
        if resized.shape[0] < 30 or resized.shape[1] < 30:
            break

        yield resized


def sliding_window(image, stepSize, windowSize):
    # slide a window across the image
    for y in xrange(0, image.shape[0], stepSize):
        for x in xrange(0, image.shape[1], stepSize):
            # yield the current window
            yield (x, y, image[y:y + windowSize[1], x:x + windowSize[0]])


# construct the argument parser and parse the arguments
ap = argparse.ArgumentParser()
ap.add_argument("-i", "--image", required=True, help="Path to the image")
ap.add_argument("-s", "--scale", type=float, default=1.5, help="scale factor size")
args = vars(ap.parse_args())

# load the image
image = cv2.imread(args["image"])
(winW, winH) = (128, 128)

orb = cv2.ORB()

# loop over the image pyramid
for resized in pyramid(image, scale=1.5):
    # loop over the sliding window for each layer of the pyramid
    for (x, y, window) in sliding_window(resized, stepSize=32, windowSize=(winW, winH)):
        # if the window does not meet our desired window size, ignore it
        if window.shape[0] != winH or window.shape[1] != winW:
            continue

        mask = np.zeros((resized.shape[0], resized.shape[1]), dtype=np.uint8)
        # mask.fill(0)
        mask[x:(x + winW), y:(y + winH)] = 1

        converted_resized = (resized * 255).astype(np.uint8)

        kp = orb.detect(converted_resized, mask)

        # compute the descriptors with ORB
        kp, des = orb.compute(converted_resized, kp)

    # clone = resized.copy()
    # cv2.rectangle(clone, (x, y), (x + winW, y + winH), (0, 255, 0), 2)
    # cv2.imshow("Window", clone)
    # cv2.waitKey(1)
    # time.sleep(0.025)

    # img2 = cv2.drawKeypoints(converted_resized,kp,color=(0,255,0), flags=0)
    # plt.imshow(img2),plt.show()

    # since we do not have a classifier, we'll just draw the window



# -*- coding: utf-8 -*-
from unittest import TestCase
from video_brain.graph_parser import ObjectFactory, Interpreter

class TestFactory(TestCase):

    def test_factory(self):

        interpreter1 = ObjectFactory.createObject(Interpreter.__name__)
        interpreter2 = ObjectFactory.createObject(Interpreter.__name__)
        interpreter3 = ObjectFactory.createObject(Interpreter.__name__)

        print(interpreter1)
        print(interpreter2)
        print(interpreter3)
        print(interpreter1 is interpreter2)
        print(interpreter1 is interpreter3)
        print(interpreter2 is interpreter3)




import MySQLdb as mdb
import unicodedata

class DbManager():

    def remove_accents(self, input_str):
        nkfd_form = unicodedata.normalize('NFKD', unicode(input_str.decode('utf-8')))
        return u"".join([c for c in nkfd_form if not unicodedata.combining(c)])

    def parse_tag(self, f, tags):

        new_tags = []
        for tag in tags:
            new_tags.append(self.remove_accents(f(tag)))

        return new_tags

    def get_all_rushs_tags(self):

        con = mdb.connect('localhost', 'root', 'castor69', 'webcastor', charset='utf8', use_unicode=True)
        cur = con.cursor()
        cur.execute('SELECT tags, media_ref FROM jos_inwicast_medias')

        str_tags = cur.fetchall()
        tags = []
        for str_tag in str_tags:
            tags = tags + [(self.parse_tag(lambda x: x.strip().lower().encode('utf-8'), str_tag[0].split(',')), str_tag[1])]

        return tags


#!/usr/bin/env python

#Data layer for video.  Change flow_frames and RGB_frames to be the path to the flow and RGB frames.

import sys
sys.path.append('/home/leo/lisa-caffe/python')
import caffe
import numpy as np
import glob
import random
from multiprocessing import Pool
from threading import Thread

flow_frames = '../resources/motion/flow_images/'
RGB_frames = '../resources/motion/frames/'
test_frames = 16 
train_frames = 16
test_buffer = 3
train_buffer = 24

def processImageCrop(im_info, transformer, flow):
  im_path = im_info[0]
  im_crop = im_info[1] 
  im_reshape = im_info[2]
  im_flip = im_info[3]
  data_in = caffe.io.load_image(im_path)
  if (data_in.shape[0] < im_reshape[0]) | (data_in.shape[1] < im_reshape[1]):
    data_in = caffe.io.resize_image(data_in, im_reshape)
  if im_flip:
    data_in = caffe.io.flip_image(data_in, 1, flow) 
    data_in = data_in[im_crop[0]:im_crop[2], im_crop[1]:im_crop[3], :] 
  processed_image = transformer.preprocess('data_in',data_in)
  return processed_image

class ImageProcessorCrop(object):
  def __init__(self, transformer, flow):
    self.transformer = transformer
    self.flow = flow
  def __call__(self, im_info):
    return processImageCrop(im_info, self.transformer, self.flow)

class sequenceGeneratorVideo(object):
  def __init__(self, buffer_size, clip_length, num_videos, video_dict, video_order):
    self.buffer_size = buffer_size
    self.clip_length = clip_length
    self.N = self.buffer_size*self.clip_length
    self.num_videos = num_videos
    self.video_dict = video_dict
    self.video_order = video_order
    self.idx = 0

  def __call__(self):
    label_r = []
    im_paths = []
    im_crop = []
    im_reshape = []  
    im_flip = []
 
    if self.idx + self.buffer_size >= self.num_videos:
      idx_list = range(self.idx, self.num_videos)
      idx_list.extend(range(0, self.buffer_size-(self.num_videos-self.idx)))
    else:
      idx_list = range(self.idx, self.idx+self.buffer_size)
    

    for i in idx_list:
      key = self.video_order[i]
      label = self.video_dict[key]['label']
      video_reshape = self.video_dict[key]['reshape']
      video_crop = self.video_dict[key]['crop']
      label_r.extend([label]*self.clip_length)

      im_reshape.extend([(video_reshape)]*self.clip_length)
      r0 = int(random.random()*(video_reshape[0] - video_crop[0]))
      r1 = int(random.random()*(video_reshape[1] - video_crop[1]))
      im_crop.extend([(r0, r1, r0+video_crop[0], r1+video_crop[1])]*self.clip_length)     
      f = random.randint(0,1)
      im_flip.extend([f]*self.clip_length)
      rand_frame = int(random.random()*(self.video_dict[key]['num_frames']-self.clip_length)+1+1)
      frames = []

      for i in range(rand_frame,rand_frame+self.clip_length):
        frames.append(self.video_dict[key]['frames'] %i)
     
      im_paths.extend(frames) 
    
    
    im_info = zip(im_paths,im_crop, im_reshape, im_flip)

    self.idx += self.buffer_size
    if self.idx >= self.num_videos:
      self.idx = self.idx - self.num_videos

    return label_r, im_info
  
def advance_batch(result, sequence_generator, image_processor, pool):
  
    label_r, im_info = sequence_generator()
    tmp = image_processor(im_info[0])
    result['data'] = pool.map(image_processor, im_info)
    result['label'] = label_r
    cm = np.ones(len(label_r))
    cm[0::16] = 0
    result['clip_markers'] = cm

class BatchAdvancer():
    def __init__(self, result, sequence_generator, image_processor, pool):
      self.result = result
      self.sequence_generator = sequence_generator
      self.image_processor = image_processor
      self.pool = pool
 
    def __call__(self):
      return advance_batch(self.result, self.sequence_generator, self.image_processor, self.pool)

class videoRead(caffe.Layer):

  def initialize(self):
    self.train_or_test = 'test'
    self.flow = False
    self.buffer_size = test_buffer  #num videos processed per batch
    self.frames = test_frames   #length of processed clip
    self.N = self.buffer_size*self.frames
    self.idx = 0
    self.channels = 3
    self.height = 227
    self.width = 227
    self.path_to_images = RGB_frames 
    self.video_list = '..resources/motion/test_split.txt'

  def setup(self, bottom, top):
    random.seed(10)
    self.initialize()
    f = open(self.video_list, 'r')
    f_lines = f.readlines()
    f.close()

    video_dict = {}
    current_line = 0
    self.video_order = []
    for ix, line in enumerate(f_lines):
      video = line.split(' ')[0]
      l = int(line.split(' ')[1])
      frames = glob.glob('%s%s/*.jpg' %(self.path_to_images, video))
      num_frames = len(frames)
      video_dict[video] = {}
      video_dict[video]['frames'] = '..' + frames[0].split('.')[2][:-4] + '%04d.jpg'
      video_dict[video]['reshape'] = (240,320)
      video_dict[video]['crop'] = (227, 227)
      video_dict[video]['num_frames'] = num_frames
      video_dict[video]['label'] = l
      self.video_order.append(video)

    self.video_dict = video_dict
    self.num_videos = len(video_dict.keys())

    #set up data transformer
    shape = (self.N, self.channels, self.height, self.width)
        
    self.transformer = caffe.io.Transformer({'data_in': shape})
    self.transformer.set_raw_scale('data_in', 255)
    if self.flow:
      image_mean = [128, 128, 128]
      self.transformer.set_is_flow('data_in', True)
    else:
      image_mean = [103.939, 116.779, 128.68]
      self.transformer.set_is_flow('data_in', False)
    channel_mean = np.zeros((3,227,227))
    for channel_index, mean_val in enumerate(image_mean):
      channel_mean[channel_index, ...] = mean_val
    self.transformer.set_mean('data_in', channel_mean)
    self.transformer.set_channel_swap('data_in', (2, 1, 0))
    self.transformer.set_transpose('data_in', (2, 0, 1))

    self.thread_result = {}
    self.thread = None
    pool_size = 24

    self.image_processor = ImageProcessorCrop(self.transformer, self.flow)
    self.sequence_generator = sequenceGeneratorVideo(self.buffer_size, self.frames, self.num_videos, self.video_dict, self.video_order)

    self.pool = Pool(processes=pool_size)
    self.batch_advancer = BatchAdvancer(self.thread_result, self.sequence_generator, self.image_processor, self.pool)
    self.dispatch_worker()
    self.top_names = ['data', 'label','clip_markers']
    print 'Outputs:', self.top_names
    if len(top) != len(self.top_names):
      raise Exception('Incorrect number of outputs (expected %d, got %d)' %
                      (len(self.top_names), len(top)))
    self.join_worker()
    for top_index, name in enumerate(self.top_names):
      if name == 'data':
        shape = (self.N, self.channels, self.height, self.width)
      elif name == 'label':
        shape = (self.N,)
      elif name == 'clip_markers':
        shape = (self.N,)
      top[top_index].reshape(*shape)

  def reshape(self, bottom, top):
    pass

  def forward(self, bottom, top):
  
    if self.thread is not None:
      self.join_worker()

    print(self.thread_result['clip_markers'])

    sys.exit('')

    #rearrange the data: The LSTM takes inputs as [video0_frame0, video1_frame0,...] but the data is currently arranged as [video0_frame0, video0_frame1, ...]
    new_result_data = [None]*len(self.thread_result['data']) 
    new_result_label = [None]*len(self.thread_result['label']) 
    new_result_cm = [None]*len(self.thread_result['clip_markers'])
    for i in range(self.frames):
      for ii in range(self.buffer_size):
        old_idx = ii*self.frames + i
        new_idx = i*self.buffer_size + ii
        new_result_data[new_idx] = self.thread_result['data'][old_idx]
        new_result_label[new_idx] = self.thread_result['label'][old_idx]
        new_result_cm[new_idx] = self.thread_result['clip_markers'][old_idx]

    for top_index, name in zip(range(len(top)), self.top_names):
      if name == 'data':
        for i in range(self.N):
          top[top_index].data[i, ...] = new_result_data[i] 
      elif name == 'label':
        top[top_index].data[...] = new_result_label
      elif name == 'clip_markers':
        top[top_index].data[...] = new_result_cm

    self.dispatch_worker()
      
  def dispatch_worker(self):
    assert self.thread is None
    self.thread = Thread(target=self.batch_advancer)
    self.thread.start()

  def join_worker(self):
    assert self.thread is not None
    self.thread.join()
    self.thread = None

  def backward(self, top, propagate_down, bottom):
    pass

class videoReadTrain_flow(videoRead):

  def initialize(self):
    self.train_or_test = 'train'
    self.flow = True
    self.buffer_size = train_buffer  #num videos processed per batch
    self.frames = train_frames   #length of processed clip
    self.N = self.buffer_size*self.frames
    self.idx = 0
    self.channels = 3
    self.height = 227
    self.width = 227
    self.path_to_images = flow_frames 
    self.video_list = 'ucf101_split1_trainVideos.txt' 

class videoReadTest_flow(videoRead):

  def initialize(self):
    self.train_or_test = 'test'
    self.flow = True
    self.buffer_size = test_buffer  #num videos processed per batch
    self.frames = test_frames   #length of processed clip
    self.N = self.buffer_size*self.frames
    self.idx = 0
    self.channels = 3
    self.height = 227
    self.width = 227
    self.path_to_images = flow_frames 
    self.video_list = '..resources/motion/test_split.txt'

class videoReadTrain_RGB(videoRead):

  def initialize(self):

    self.train_or_test = 'train'
    self.flow = False
    self.buffer_size = train_buffer  #num videos processed per batch
    self.frames = train_frames   #length of processed clip
    self.N = self.buffer_size*self.frames
    self.idx = 0
    self.channels = 3
    self.height = 227
    self.width = 227
    self.path_to_images = RGB_frames
    self.video_list = '../resources/motion/train_split.txt'

class videoReadTest_RGB(videoRead):

  def initialize(self):
    self.train_or_test = 'test'
    self.flow = False
    self.buffer_size = test_buffer  #num videos processed per batch
    self.frames = test_frames   #length of processed clip
    self.N = self.buffer_size*self.frames
    self.idx = 0
    self.channels = 3
    self.height = 227
    self.width = 227
    self.path_to_images = RGB_frames 
    self.video_list = '../resources/motion/test_split.txt'



import ConfigParser
import os
config = ConfigParser.ConfigParser()

# config.readfp(open(__file__.replace('pyc', 'ini'), 'r'))
config.readfp(open(os.path.join(os.path.dirname(__file__), 'settings.ini'), 'r'))

caffe_root = config.get('library', 'caffe_root')
apollocaffe_root = config.get('library', 'apollocaffe_root')

key_frames_rushs_path = config.get('data', 'key_frames_rushs_path')


import os
import shutil
from distutils.dir_util import copy_tree


class OsManager:

    def create_working_directory(self, working_directory, model, labels):

        # if os.path.exists(working_directory):
        #     shutil.rmtree(working_directory)
        # os.makedirs(working_directory)
        fromDirectory = os.path.join(os.path.dirname(__file__), '../resources/networks/' + model)
        toDirectory = working_directory + 'network/'
        copy_tree(fromDirectory, toDirectory)
        os.remove(toDirectory + 'weights.caffemodel')

        os.rename(toDirectory + 'train_val.prototxt', toDirectory + '/old_train_val.prototxt')
        new_train_val = open(toDirectory + '/train_val.prototxt', 'w')
        with open(toDirectory + '/old_train_val.prototxt', 'r') as train_val:
            for line in train_val:
                if 'dynamic_num_output' in line:
                    new_train_val.write(line.replace('dynamic_num_output', str(len(labels))))
                elif '../' in line:
                    new_train_val.write(line.replace('../', working_directory))
                else:
                    new_train_val.write(line)

        new_train_val.close()
        os.remove(toDirectory + '/old_train_val.prototxt')

        os.rename(toDirectory + 'valnet.prototxt', toDirectory + '/old_valnet.prototxt')
        new_valnet = open(toDirectory + '/valnet.prototxt', 'w')
        with open(toDirectory + '/old_valnet.prototxt', 'r') as valnet:
            for line in valnet:
                if 'dynamic_num_output' in line:
                    new_valnet.write(line.replace('dynamic_num_output', str(len(labels))))
                elif '../' in line:
                    new_valnet.write(line.replace('../', working_directory))
                else:
                    new_valnet.write(line)

        new_valnet.close()
        os.remove(toDirectory + '/old_valnet.prototxt')


# -*- coding: utf-8 -*-
import unittest
from unittest import TestCase
from video_brain.graph_parser.graph_browser import GraphBrowser
import time

class TestGraphBrowser(TestCase):

    def test_fit(self):

        graph_browser = GraphBrowser()
        # graph_browser.fit('/home/leo/Desktop/graph/', 'deep_residual')

        graph_browser.fit('/home/leo/Desktop/graph_deep_residual/', 'deep_residual')
        # graph_browser.fit('/home/leo/Desktop/graph_alexnet/', 'alexnet')

if __name__ == '__main__':
    unittest.main()


from PIL import TarIO, Image, JpegImagePlugin
from .utils import remove_accents
from .interpreter import Interpreter
from .factory import ObjectFactory
from smb.SMBHandler import SMBHandler
import MySQLdb as mdb
import tarfile
import cv2
import glob
import numpy as np
import random
import os
import urllib2
import sys
import pickle
import fnmatch
import settings
import itertools

class DataProvider():

    def __init__(self, task):

        self.task = task

    def get_data_from_rushs(self, labels_synonyms, labels):

        interpreter = ObjectFactory.createObject(Interpreter.__name__)

        con = mdb.connect('localhost', 'root', 'castor69', 'webcastor', charset='utf8', use_unicode=True)
        cur = con.cursor()

        if self.task == 'themes':

            cur.execute('SELECT media_ref, themes FROM jos_inwicast_medias')

            refs = cur.fetchall()

            X = []
            y = []
            for ref in refs:

                media_ref = ref[0]
                label = remove_accents(ref[1].lower())

                if label != '' and label in labels_synonyms.keys():

                    medianames = glob.glob('../../images/key_frames_rushs/' + media_ref + '*.JPEG')

                    for medianame in medianames:

                        img = cv2.imread(medianame)
                        img = cv2.resize(img, (227, 227))

                        X.append(img)
                        y.append(label)

            targets, y = np.unique(y, return_inverse=True)

        elif self.task == 'tags':

            key_frames_names = glob.glob(settings.key_frames_rushs_path + '*.JPEG')
            random.shuffle(key_frames_names)

            for key_frames_name in key_frames_names:

                media_ref = key_frames_name.split('/')[-1].split('_')[0]

                cur.execute('SELECT tags FROM jos_inwicast_medias WHERE media_ref = %s', (media_ref,))

                tags = cur.fetchone()[0]
                tags = tags.split(',')
                tags = [interpreter.translate(tag.strip().lower()) for tag in tags]
                tags = [tag for tag in tags if tag is not None]

                all_tags_synonyms = []
                for tag in tags:

                    synonyms = interpreter.get_synonyms(tag)
                    all_tags_synonyms = all_tags_synonyms + synonyms

                target = []
                for label in labels:

                    label_synonyms = labels_synonyms[label]

                    label_is_hypernyms = False
                    for tag in tags:
                        if interpreter.is_hypernym(label, tag) is True:
                            label_is_hypernyms = True
                            break

                    if len(set(all_tags_synonyms).intersection(label_synonyms)) != 0 or label_is_hypernyms is True:
                        target.append(1)
                    else:
                        target.append(0)

                if 1 in target:

                    img = cv2.imread(key_frames_name)
                    img = cv2.resize(img, (227, 227))

                    yield(img, target)

    def get_data_from_search_engine(self, labels, search_engine=None):

        X = []
        y = []
        for root, dirnames, filenames in os.walk('../../images/bing'):

            for filename in fnmatch.filter(filenames, '*.jpg'):

                label = root.split('/')[-1]

                if label in labels.values():

                    img = cv2.imread(os.path.join(root, filename))

                    if img != None:
                        img = cv2.resize(img, (227, 227))
                        X.append(img)
                        y.append(label)

        return X,y

    def get_data_from_dataset(self, labels_synonyms, labels):

        for img, target in itertools.chain(self.get_data_from_imagenet(labels_synonyms, labels)):
            yield(img, target)

    def get_data_from_imagenet(self, labels_synonyms, labels):

        interpreter = ObjectFactory.createObject(Interpreter.__name__)

        wnid_mapping = pickle.load(open(os.path.join(os.path.dirname(__file__), '../resources/nlp/wnid_mapping.pkl'), 'r'))

        cpt = 1
        for wnid, words in wnid_mapping.iteritems():

            if cpt % 30000 == 0:
                break
                print(str(cpt) + ' synsets processed')

            for key, label in enumerate(labels):

                for word in words:

                    if interpreter.is_hypernym(label, word):

                        str_to_write = label + ' is hypernim of ' + word + ' (' + str(wnid) + ')'
                        # print(str_to_write)

                        try:
                            director = urllib2.build_opener(SMBHandler)
                            fh = director.open(
                                'smb://Production:Castor69@synology.local/projets_video/wanaclip_brain/imagenet/'+wnid+'.tar')

                            f = open('/home/leo/Desktop/'+wnid+'.tar', 'wb')
                            f.write(fh.read())
                            f.close()

                        except (urllib2.URLError,):
                            pass
                            # print(sys.exc_info()[0])
                            # print(sys.exc_info()[1])

                        if os.path.exists('/home/leo/Desktop/'+wnid+'.tar'):

                            try:
                                tar = tarfile.open(mode="r", fileobj=file('/home/leo/Desktop/'+wnid+'.tar'))

                                for img_name in tar.getnames():

                                    fp = TarIO.TarIO('/home/leo/Desktop/'+wnid+'.tar', img_name)
                                    img = Image.open(fp)

                                    if isinstance(img, JpegImagePlugin.JpegImageFile):

                                        img = cv2.resize(np.array(img), (227, 227))

                                        target = np.zeros((len(labels)))
                                        target[key] = 1

                                        yield(img, target)

                                os.remove('/home/leo/Desktop/'+wnid+'.tar')

                            except tarfile.ReadError:
                                print(sys.exc_info()[0])
                                print(sys.exc_info()[1])

            cpt += 1

    def get_images(self, labels_synonyms, labels):

        # for img, target in itertools.chain(self.get_data_from_rushs(labels_synonyms, labels),
        #                                    self.get_data_from_dataset(labels_synonyms, labels)):
        #     yield (img, target)

        for img, target in itertools.chain(self.get_data_from_rushs(labels_synonyms, labels),):
            yield (img, target)



from .data_provider import DataProvider
from .image_processer import ImageProcesser
from .os_manager import OsManager
from .network_trainer import NetworkTrainer
from .network_evaluator import NetworkEvaluator
from .interpreter import Interpreter
from .factory import ObjectFactory
from pygraphml import GraphMLParser
from distutils.dir_util import copy_tree
from multiprocessing import Pool
import numpy as np
import pickle
import os
import sys
import shutil
import warnings
import settings
import glob
sys.path.insert(0, settings.caffe_root + 'python')
import caffe
import time

class GraphBrowser():

    def training(self, nodes, network_evaluator, user_directory, model):

        interpreter = ObjectFactory.createObject(Interpreter.__name__)
        network_trainer = NetworkTrainer('tags')
        os_manager = OsManager()

        labels = [node['name'].lower() for node in nodes]
        print(labels)

        synonyms = dict()
        for node in nodes:

            node_name = node['name'].lower()

            node_synonyms = interpreter.get_synonyms(node_name)

            for child in self.get_all_children(node):
                child_name = child['name'].lower()
                child_synonyms = interpreter.get_synonyms(child_name)

                node_synonyms = node_synonyms + child_synonyms

            synonyms[node_name] = node_synonyms

        working_directory = user_directory + '_'.join(labels) + '/'
        os_manager.create_working_directory(working_directory, model, labels)
        p = network_trainer.train_network(working_directory, synonyms, labels, model)
        if p is not None:
            network_evaluator.add_training((p, working_directory, labels))

    def inference(self, nodes, image):

        # if 'nature' in [node['name'].lower() for node in nodes]:
        #     nodes.reverse()
        labels = [node['name'].lower() for node in nodes]

        weight_file = os.path.join(os.path.dirname(__file__), '../resources/tags/'+'_'.join(labels)+'/weights.caffemodel')

        net = caffe.Net(os.path.join(os.path.dirname(__file__), '../resources/tags/'+'_'.join(labels)+'/deploy.prototxt'),
                        weight_file,
                        caffe.TEST)
        # threshold_probas = pickle.load(
        #     open(os.path.dirname(os.path.realpath(__file__)) + '/../resources/tags/' + '_'.join(labels) + '/thresholds.pkl',
        #          'r'))

        img = image[:, :, ::-1]

        transformer = caffe.io.Transformer({'data': net.blobs['data'].data.shape})
        transformer.set_transpose('data', (2, 0, 1))
        transformed_image = transformer.preprocess('data', img)

        transformed_image = np.reshape(transformed_image, (1, 3, 227, 227))

        net.blobs['data'].reshape(*transformed_image.shape)
        net.blobs['data'].data[...] = transformed_image

        net.forward()

        probas = np.absolute(net.blobs['proba'].data[0, ...])

        return probas

    def get_all_children(self, node):

        children = [node]
        if len(node.children()) != 0:
            for child in node.children():
                children = children + self.get_all_children(child)
        else:
            return children

        return children

    def browse_graph(self, nodes, network_evaluator, user_directory, phase, model, image=None, probas_labels=None):

        node_names = [node['name'].lower() for node in nodes]

        # if phase == 'fit' and 'nature' in node_names:
        # if phase == 'fit' and 'building' in node_names:
        if phase == 'fit' and 'sand' in node_names:
        # if phase == 'fit':
            self.training(nodes, network_evaluator, user_directory, model)
        # elif phase == 'predict' and ('beach' in node_names or 'nature' in node_names):
        elif phase == 'predict':
            probas = self.inference(nodes, image)
            probas_labels.append((node_names, probas))

        for key, node in enumerate(nodes):

                childs = node.children()

                if len(childs) != 0:
                    proba_labels = self.browse_graph(childs, network_evaluator, user_directory, phase, model, image, probas_labels)
                    probas_labels.append(proba_labels)
                else:
                    if 'nature' in node_names:
                        return (node_names, probas)
                    else:
                        return (node_names, None)

        return proba_labels

    def fit(self, user_directory, model='deep_residual'):

        parser = GraphMLParser()
        graph = parser.parse(os.path.join(os.path.dirname(__file__), '../resources/onto.graphml'))

        network_evaluator = NetworkEvaluator()
        network_evaluator.monitor()

        for node in graph.nodes():
            if node['name'] == 'Entity':
                root = node

        self.browse_graph(root.children(), network_evaluator, user_directory, 'fit', model, None, [])

    def predict(self, image):

        parser = GraphMLParser()
        graph = parser.parse(os.path.join(os.path.dirname(__file__), '../resources/onto.graphml'))

        for node in graph.nodes():
            if node['name'] == 'Entity':
                root = node

        probas_labels = []
        self.browse_graph(root.children(), None, None, 'predict', None, image, probas_labels)

        return probas_labels


from .data_provider import DataProvider
from .image_processer import ImageProcesser
from .factory import ObjectFactory
from collections import defaultdict
from itertools import islice
from subprocess import call, Popen
import matplotlib.pyplot as plt
import os
import lmdb
import settings
import random
import sys
import itertools
import numpy as np
import time

sys.path.insert(0, settings.caffe_root + 'python')
import caffe as master_caffe

class NetworkTrainer():

    def __init__(self, task):

        self.task = task

    def set_db(self, working_directory, images_targets, labels, synonyms):

        if not os.path.exists(working_directory + 'database/train_rush_images'):
            os.makedirs(working_directory + 'database/train_rush_images')
        if not os.path.exists(working_directory + 'database/train_rush_labels'):
            os.makedirs(working_directory + 'database/train_rush_labels')
        if not os.path.exists(working_directory + 'database/val_rush_images'):
            os.makedirs(working_directory + 'database/val_rush_images')
        if not os.path.exists(working_directory + 'database/val_rush_labels'):
            os.makedirs(working_directory + 'database/val_rush_labels')

        map_size = 1000000 * 10 * 1000000

        train_env_images = lmdb.open(working_directory + 'database/train_rush_images', map_size=map_size)
        train_env_targets = lmdb.open(working_directory + 'database/train_rush_labels', map_size=500000000)

        val_env_images = lmdb.open(working_directory + 'database/val_rush_images', map_size=map_size)
        val_env_targets = lmdb.open(working_directory + 'database/val_rush_labels', map_size=500000000)

        train_id_img = 0
        val_id_img = 0
        train_nb_labels = defaultdict(int)
        val_nb_labels = defaultdict(int)
        all_keys = []
        with train_env_images.begin(write=True) as train_txn_images:
            with train_env_targets.begin(write=True) as train_txn_targets:
                with val_env_images.begin(write=True) as val_txn_images:
                    with val_env_targets.begin(write=True) as val_txn_targets:

                        for the_key, imgs_target in enumerate(images_targets):

                            imgs = imgs_target[0:4]
                            target = imgs_target[4]

                            keys = np.where(np.array(target) == 1)
                            all_keys = all_keys + [key for key in keys[0]]

                            id_labels = np.where(np.array(target) == 1)
                            for id_label in id_labels[0]:

                                if the_key % 6 == 0:
                                    val_nb_labels[labels[id_label]] += 4
                                else:
                                    train_nb_labels[labels[id_label]] += 4

                            im = np.array(target)  # or load whatever ndarray you need

                            if self.task == 'themes':
                                im = np.expand_dims(im, axis=0)
                            im = np.expand_dims(im, axis=0)
                            im = np.expand_dims(im, axis=0)
                            im_dat = master_caffe.io.array_to_datum(im)

                            datum = master_caffe.proto.caffe_pb2.Datum()

                            datum.channels = imgs[0].shape[2]
                            datum.height = imgs[0].shape[0]
                            datum.width = imgs[0].shape[1]
                            datum.label = 0

                            for img in imgs:

                                img = img[:, :, ::-1]
                                img = img.transpose((2, 0, 1))
                                datum.data = img.tobytes()

                                if the_key % 6 == 0:
                                    str_id = '{:08}'.format(val_id_img)
                                    val_txn_images.put(str_id.encode('ascii'), datum.SerializeToString())
                                    val_txn_targets.put(str_id.encode('ascii'), im_dat.SerializeToString())
                                    val_id_img += 1
                                else:
                                    str_id = '{:08}'.format(train_id_img)
                                    train_txn_images.put(str_id.encode('ascii'), datum.SerializeToString())
                                    train_txn_targets.put(str_id.encode('ascii'), im_dat.SerializeToString())
                                    train_id_img += 1

        train_env_images.close()
        train_env_targets.close()
        val_env_images.close()
        val_env_targets.close()

        with open(working_directory + 'database/train_stats.txt', 'a') as report:

            report.write('Total number of images in the training set : ' + str(train_id_img) + '\n')
            for label in labels:
                report.write(
                    'Number of images for the class ' + str(label) + ' : ' + str(train_nb_labels[label]) + '\n')

        with open(working_directory + 'database/val_stats.txt', 'a') as report:

            report.write('Total number of images in the validation set : ' + str(val_id_img) + '\n')
            for label in labels:
                report.write(
                    'Number of images for the class ' + str(label) + ' : ' + str(val_nb_labels[label]) + '\n')

        return np.delete(synonyms.keys(), np.unique(all_keys))

    def train_network(self, working_directory, synonyms, labels, model):

        data_provider = DataProvider('tags')
        image_processer = ImageProcesser()

        augmented_images_targets_iterable = image_processer.data_augmentation(data_provider.get_images(synonyms, labels), working_directory)

        missing_labels = self.set_db(working_directory, augmented_images_targets_iterable, labels, synonyms)

        missing_labels = np.unique(missing_labels)

        if len(missing_labels) != 0:

            with open(working_directory + '/missing_labels.txt', 'w') as file:
                file.write(
                    "Some labels are not represented in the images : " + ' - '.join(missing_labels).encode('utf-8'))
                file.write('\n \n')

                for missing_label in missing_labels:
                    file.write(missing_label.encode('utf-8') + ' : \n')
                    file.write(' - '.join(synonyms[missing_label]).encode('utf-8') + '\n \n')

            p = None

        else:

            call([settings.caffe_root + 'build/tools/compute_image_mean',
                   working_directory + 'database/train_rush_images',
                   working_directory + 'database/mean.binaryproto'])

            solver_file = working_directory + 'network/solver.prototxt'

            if not os.path.exists(working_directory + 'training'):
                os.makedirs(working_directory + 'training')

            temp_solver_name = solver_file.split('.')[0] + '_temp.prototxt'
            temp_solver = open(temp_solver_name, 'w')
            with open(solver_file, 'r') as solver:
                for line in solver:
                    if 'snapshot_prefix' in line:
                        temp_solver.write('snapshot_prefix: "' + working_directory + 'training/snapshot" \n')
                    elif './' in line:
                        temp_solver.write(line.replace('./',  working_directory + 'network/'))
                    else:
                        temp_solver.write(line)
            temp_solver.close()

            with open(working_directory + "/output.txt", "wb") as f:
                p = Popen([settings.caffe_root + "build/tools/caffe", 'train', '--solver',
                      temp_solver_name, '--weights', os.path.join(os.path.dirname(__file__), '../resources/networks/'+model+'/weights.caffemodel') ],
                     stderr=f)

        return p


from motion_detection import motion_detection
from clustering import cluster_video
import matplotlib.pyplot as plt
import sys
import cv2
import glob
import logging
import skvideo.io

# logging.basicConfig(filename='../logs/key_frames_log.txt', level=logging.DEBUG)
from skvideo.io import vread

def extract_key_frames(videoname):

    # cap = cv2.VideoCapture(videoname)
    ext = videoname.split('.')[-1]

    # if ext == 'f4v' or ext == 'mp4':

    try:
        cap = vread(videoname, backend='libav')

    except RuntimeError:
        print('Runtime Error')

        # try:
        #     movement = motion_detection(cap)
        # except:
        #     movement = ''
        #     logging.exception('motion detection : ' + str(videoname))

        # if movement == 'plan fixe':

    key_frames = []

    # nb_frames = cap.get(7)
    nb_frames = cap.shape[0]

    # cap.set(1,int(nb_frames/3))
    # ret, frame = cap.read()
    frame = cap[int(nb_frames / 3), :, :, :]
    key_frames.append(frame)

    # cap.set(1,(2*nb_frames)/3)
    # ret, frame = cap.read()
    frame = cap[int((2 * nb_frames) / 3), :, :, :]
    key_frames.append(frame)

        # else:

            # key_frames = []
            # cap.set(1,0)
            # try:
            #     key_frames = cluster_video(cap)
            # except:
            #     logging.exception('cluster video : ' + str(videoname))
    # else:
    #     key_frames = []

    # cv2.destroyAllWindows()
    # cap.release()

    return key_frames


def get_key_frames(rushnames):

    videos = []
    for rushname in rushnames:
        key_frames = extract_key_frames(rushname)
        videos.append(key_frames)

    return videos


def process_specific_frames():
    rushs_frames = get_key_frames(sys.argv[1:])


# write_frames(rushs_frames, sys.argv[1:])

def process_all_frames():
    videonames = glob.glob('../../../rushs/*')

    all_images = glob.glob('../../../images/*')
    all_images = [name.split('/')[-1].split('.')[0].split('_')[0] for name in all_images]

    key_frames = []

    for videoname in videonames:

        name = videoname.split('/')[-1].split('.')[0].split('_')[0]

        if name not in all_images:
            rushs_frames = get_key_frames([videoname])
            write_frames(rushs_frames, [videoname])
        # key_frames.append(rushs_frames)

    return key_frames


def write_frames(rushs_frames, videonames):
    for rush, name in zip(rushs_frames, videonames):
        name = name.split('/')[-1].split('.')[0].split('_')[0]
        for i, frame in enumerate(rush):
            # plt.imshow(frame), plt.show()
            cv2.imwrite('../../../images/rushs/' + name + '_' + str(i) + '.png', frame)

# process_specific_frames()
# process_all_frames()



import socket
import urllib2
from _ssl import SSLError

from gensim.models.word2vec import Word2Vec
from nltk.corpus import wordnet as wn
import numpy as np
import pickle
import os
import sys
import goslate
import time

class Interpreter:

    def __init__(self):

        model = Word2Vec.load_word2vec_format(os.path.dirname(os.path.realpath(__file__))+'/../resources/nlp/GoogleNews-vectors-negative300.bin', binary=True)
        model.init_sims(replace=True)
        self.word2vec_model = model
        self.synonyms = {}
        self.dictionary = pickle.load(open(os.path.join(os.path.dirname(__file__), '../resources/nlp/dictionary.pkl')))

    def get_synonyms(self, word):

        if word not in self.synonyms.keys():
            try:
                synonyms = self.word2vec_model.most_similar(positive=[word])
                synonyms = [similar_name[0].lower() for similar_name in synonyms]
            except KeyError:
                synonyms = []

            try:
                word_synset = wn.synset(word + '.n.01')
                for lemma in word_synset.lemmas():
                    synonyms.append(lemma.name())
            except:
                pass

            self.synonyms[word] = synonyms
        else:
            synonyms = self.synonyms[word]

        synonyms.append(word.lower())

        return list(np.unique(synonyms))

    def is_hypernym(self, label, tag):

        try:
            label_synset = wn.synset(label + '.n.01')
            tag_synset = wn.synset(tag + '.n.01')
        except:
            label_synset = None
            tag_synset = None

        if label_synset is not None and tag_synset is not None:
            tag_hypernyms = self.all_hypernyms(tag_synset)

            if len(tag_hypernyms.intersection([label_synset])) != 0:
                return True
            else:
                return False

    def semantic_similarity(self, word1, word2):
        pass

    def translate(self, word):

        if word in self.dictionary.keys():
            return self.dictionary[word].lower()
        else:

            gs = goslate.Goslate(service_urls=['https://translate.google.de'])

            try:
                tag_en = gs.translate(word, 'en')

                return tag_en

            except (socket.timeout, urllib2.HTTPError, urllib2.URLError, SSLError, TypeError):
                # print(sys.exc_info()[0])
                # print(sys.exc_info()[1])
                with open('/home/leo/Desktop/not_trad.txt', 'a') as file:
                    file.write(word.encode('utf-8') + '\n')

                return None




    def _recurse_all_hypernyms(self, synset, all_hypernyms):
        synset_hypernyms = synset.hypernyms()
        if synset_hypernyms:
            all_hypernyms += synset_hypernyms
            for hypernym in synset_hypernyms:
                self._recurse_all_hypernyms(hypernym, all_hypernyms)

    def all_hypernyms(self, synset):
        hypernyms = []
        self._recurse_all_hypernyms(synset, hypernyms)
        return set(hypernyms)


    class Factory:

        interpreter = None

        def create(self):

            if not Interpreter.Factory.interpreter:
                Interpreter.Factory.interpreter = Interpreter()

            return self.interpreter


import numpy as np
import cv2
import os
import sys
import time

class ImageProcesser():

    def data_augmentation(self, images_targets_iterable, working_directory):

        if not os.path.exists(working_directory + 'database/'):
            os.makedirs(working_directory + 'database/')

        with open(working_directory + 'database/data_transformation.txt', 'w') as report:
            report.write('Flipped Image : True \n')
            report.write('Affine Transformation : True (*2) \n')

        for img, target in images_targets_iterable:

            try:
                flipped_img = cv2.flip(img, 1)

                rows, cols, ch = img.shape
                pts1 = np.float32([[50, 50], [200, 50], [50, 200]])
                pts2 = np.float32([[10, 100], [200, 50], [100, 250]])
                pts3 = np.float32([[200, 200], [70, 70], [200, 50]])
                pts4 = np.float32([[150, 250], [70, 70], [250, 0]])
                M1 = cv2.getAffineTransform(pts1, pts2)
                M2 = cv2.getAffineTransform(pts3, pts4)
                affine_img_1 = cv2.warpAffine(img, M1, (cols, rows))
                affine_img_2 = cv2.warpAffine(img, M2, (cols, rows))

                yield (img, flipped_img, affine_img_1, affine_img_2, target)

            except ValueError:
                pass
                # print(sys.exc_info()[0])
                # print(sys.exc_info()[1])





# -*- coding: utf-8 -*-
import MySQLdb as mdb
from sklearn.svm import SVC
import numpy as np
from sklearn.cross_validation import train_test_split
from sklearn.metrics import classification_report
import sys
import pickle
from collections import defaultdict


def themes_classification():
    con = mdb.connect('localhost', 'root', 'castor69', 'webcastor', charset='utf8', use_unicode=True)
    cur = con.cursor()

    cur.execute("SELECT caffe_features, media FROM features WHERE more_images = 1")

    caffe_features = []
    labels = []
    for triplet in cur.fetchall():

        strfeats = triplet[0]
        strfeats = strfeats.rstrip(']')
        strfeats = strfeats.lstrip('[')
        strfeats = strfeats.split()
        feats = [float(feat) for feat in strfeats]
        feats = np.array(feats)

        print(feats)
        sys.exit('')

        # print(triplet[1])
        cur.execute("SELECT themes FROM jos_inwicast_medias WHERE media_ref = %s", (triplet[1],))

        label = cur.fetchone()

        if label:
            if label[0] != '':
                labels.append(label[0])
                caffe_features.append(feats)

    cur.execute("SELECT caffe_features, media FROM features WHERE more_images is null")

    for triplet in cur.fetchall():

        strfeats = triplet[0]
        strfeats = strfeats.rstrip(']')
        strfeats = strfeats.lstrip('[')
        strfeats = strfeats.split()
        feats = [float(feat) for feat in strfeats]
        feats = np.array(feats)

        cur.execute("SELECT themes FROM jos_inwicast_medias WHERE media_ref = %s", (triplet[1],))
        label = cur.fetchone()

        # print(label)

        if label:
            label = label[0]
            if label == 'Campagne / Zone rural' or label == 'Montagne' or label == 'Ville / Zone urbaine':
                labels.append(label)
                caffe_features.append(feats)

    labels = np.array(labels)
    caffe_features = np.array(caffe_features)

    # all_labels, counts = np.unique(labels, return_counts=True)

    X_train, X_test, y_train, y_test = train_test_split(caffe_features, labels,
                                                        test_size=0.33, random_state=42)

    # clf = SVC()


    # all_labels, counts = np.unique(y_train, return_counts=True)
    # print(all_labels)
    # print(counts)

    # print('fit model')
    # clf.fit(X_train, y_train)

    clf = pickle.load(open('model.p', 'r'))

    y_test_pred = clf.predict(X_test)

    # pickle.dump(clf, open('model.p', 'w'))
    # pickle.dump(y_test_pred, open('y_test_pred.p', 'w'))
    # pickle.dump(y_test, open('y_test.p', 'w'))

    labels_not_predicted = defaultdict(int)
    for label in np.unique(y_test):
        if label not in np.unique(y_test_pred):
            labels_not_predicted[label] += 1
            ids = np.where(y_test == label)
            y_test = np.delete(y_test, ids)
            y_test_pred = np.delete(y_test_pred, ids)

    themes_classification_report = classification_report(y_test, y_test_pred)
    print(themes_classification_report)
    f = open('themes_classification_report.txt', 'w')
    f.write(themes_classification_report.encode('utf-8'))
    f.close()


themes_classification()



import MySQLdb as mdb
import sys
import numpy as np
import pickle
import random
from sklearn.cross_validation import train_test_split
from sklearn.svm import SVC
from sklearn.metrics import classification_report


def get_positive_samples(cur, tag):
    cur.execute("SELECT media_ref FROM jos_inwicast_medias WHERE tags LIKE '%" + str(tag) + "%' ")

    rushs = cur.fetchall()

    pos_features = []
    for media_ref in rushs:

        media = media_ref[0]

        cur.execute("SELECT caffe_features FROM features WHERE media = %s", (media,))

        features = cur.fetchall()

        for feats in features:
            feats = feats[0]
            feats = feats.rstrip(']')
            feats = feats.lstrip('[')
            feats = feats.split()
            feats = [float(feat) for feat in feats]

            pos_features.append(feats)

    pos_features = np.array(pos_features)

    pickle.dump(pos_features, open('pos_features.p', 'w'))

    return pos_features


def get_negative_samples(cur, tag, size):
    cur.execute("SELECT media_ref FROM jos_inwicast_medias WHERE tags NOT LIKE '%" + str(tag) + "%' ")

    rushs = cur.fetchall()
    rushs = [rush[0] for rush in rushs]
    random.shuffle(rushs)

    rushs = rushs[0:size]

    neg_features = []
    for media_ref in rushs:

        media = media_ref

        cur.execute("SELECT caffe_features FROM features WHERE media = %s", (media,))

        features = cur.fetchall()

        for feats in features:
            feats = feats[0]
            feats = feats.rstrip(']')
            feats = feats.lstrip('[')
            feats = feats.split()
            feats = [float(feat) for feat in feats]

            neg_features.append(feats)

    neg_features = np.array(neg_features)

    pickle.dump(neg_features, open('neg_features.p', 'w'))

    return neg_features


def entity_recognition(tag):
    con = mdb.connect('localhost', 'leo', 'castor69', 'webcastor', charset='utf8', use_unicode=True)
    cur = con.cursor()

    # pos_features = get_positive_samples(cur, tag)
    # pos_features = pickle.load(open('pos_features.p', 'r'))
    # print(pos_features.shape)

    neg_features = get_negative_samples(cur, tag, pos_features.shape[0])
    # neg_features = pickle.load(open('neg_features.p', 'r'))
    print(neg_features.shape)

    features = np.concatenate(pos_features, neg_features)

    print(features.shape)

    pos_labels = np.empty(pos_features.shape[0])
    pos_labels.fill(1)

    neg_labels = np.empty(neg_features.shape[0])
    neg_labels.fill(0)

    labels = np.concatenate(pos_labels, neg_labels)

    print(labels.shape)

    training_data = list(zip(features, labels))
    random.shuffle(training_data)
    features, labels = zip(*training_data)

    print(features.shape)
    X_train, X_test, y_train, y_test = train_test_split(features, labels, test_size=0.33, random_state=42)

    clf = SVC()
    clf.fit(X_train, y_train)
    y_pred = clf.predict(X_test)

    pickle.dump(clf, open('model.p', 'w'))

    entity_recognition_report = classification_report(y_test, y_pred)
    f = open('entity_recognition_report.txt', 'w')
    f.write(entity_recognition_report)
    f.close()


tag = sys.argv[1]
entity_recognition(tag)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(name='video_brain',
      version='0.1',
      description='Deep Learning algorithms for video processing tasks',
      # url='http://github.com/storborg/funniest',
      author='Léo Vetter',
      author_email='vetter.leo@gmail.com',
      license='MIT',
      packages=find_packages(),
      zip_safe=False,
      test_suite='nose.collector',
      tests_require=['nose'],
      include_package_data=True,
      )


#!/usr/bin/env python
# -*- coding: utf-8 -*
import MySQLdb as mdb
from subprocess import call
import numpy as np
import os
import glob
import sys
import pickle
import inspect

sys.path.append('/home/leo/lisa-caffe/python')
import caffe


def initialize_transformer(image_mean, is_flow):
    shape = (10 * 16, 3, 227, 227)
    transformer = caffe.io.Transformer({'data': shape})
    channel_mean = np.zeros((3, 227, 227))
    for channel_index, mean_val in enumerate(image_mean):
        channel_mean[channel_index, ...] = mean_val
    transformer.set_mean('data', channel_mean)
    transformer.set_raw_scale('data', 255)
    transformer.set_channel_swap('data', (2, 1, 0))
    transformer.set_transpose('data', (2, 0, 1))
    transformer.set_is_flow('data', is_flow)
    return transformer


def LRCN_classify_video(frames, net, transformer, is_flow):
    clip_length = 16
    offset = 8
    input_images = []
    for im in frames:
        input_im = caffe.io.load_image(im)
        if (input_im.shape[0] < 240):
            input_im = caffe.io.resize_image(input_im, (240, 320))
        input_images.append(input_im)
    vid_length = len(input_images)
    input_data = []
    for i in range(0, vid_length, offset):
        if (i + clip_length) < vid_length:
            input_data.extend(input_images[i:i + clip_length])
        else:  # video may not be divisible by clip_length
            input_data.extend(input_images[-clip_length:])
    output_predictions = np.zeros((len(input_data), 8))
    for i in range(0, len(input_data), clip_length):
        clip_input = input_data[i:i + clip_length]
        clip_input = caffe.io.oversample(clip_input, [227, 227])
        clip_clip_markers = np.ones((clip_input.shape[0], 1, 1, 1))
        clip_clip_markers[0:10, :, :, :] = 0
        #    if is_flow:  #need to negate the values when mirroring
        #      clip_input[5:,:,:,0] = 1 - clip_input[5:,:,:,0]
        caffe_in = np.zeros(np.array(clip_input.shape)[[0, 3, 1, 2]], dtype=np.float32)
        for ix, inputs in enumerate(clip_input):
            caffe_in[ix] = transformer.preprocess('data', inputs)
        out = net.forward_all(data=caffe_in, clip_markers=np.array(clip_clip_markers))
        output_predictions[i:i + clip_length] = np.mean(out['probs'], 1)
    return np.mean(output_predictions, 0).argmax(), output_predictions


def lrcn_motion_predict(user_directory):
    net = caffe.Net(os.path.dirname(os.path.realpath(__file__)) + '/../resources/motion/deploy_lstm.prototxt',
                    os.path.dirname(os.path.realpath(__file__)) + '/../resources/motion/weights.caffemodel',
                    caffe.TEST)

    ucf_mean_RGB = np.zeros((3, 1, 1))
    ucf_mean_RGB[0, :, :] = 103.939
    ucf_mean_RGB[1, :, :] = 116.779
    ucf_mean_RGB[2, :, :] = 128.68

    transformer_RGB = initialize_transformer(ucf_mean_RGB, False)
    RGB_frames = glob.glob('../resources/motion/frames/AF-AFR-01-001_500k/*.jpg')

    class_RGB_LRCN, predictions_RGB_LRCN = LRCN_classify_video(RGB_frames, net, transformer_RGB, False)

    labels = pickle.load(open(os.path.join(os.path.dirname(__file__), '../resources/motion/labels.pkl'), 'r'))

    print(class_RGB_LRCN)
    print "RGB LRCN model classified video as: %s.\n" % (labels[class_RGB_LRCN])


def lrcn_motion_training(user_directory):
    script_name = os.path.join(os.path.dirname(__file__), '../resources/motion/run_lstm_RGB.sh')
    call([script_name], shell=True)


# lrcn_motion_training('/home/leo/Desktop/lrcn_motion')
lrcn_motion_predict('/home/leo/Desktop/lrcn_motion')



import unittest
from video_brain.graph_parser.factory import ObjectFactory
from video_brain.graph_parser.interpreter import Interpreter

class TestInterpreter(unittest.TestCase):

    def setUp(self):
        self.interpreter = ObjectFactory.createObject(Interpreter.__name__)

    def test_is_hypernym(self):

        self.assertTrue(self.interpreter.is_hypernym('person', 'indian'))
        self.assertFalse(self.interpreter.is_hypernym('peanut', 'dog'))

    def test_translate(self):

        self.assertEqual('dog', self.interpreter.translate('chien'))

if __name__ == '__main__':
    # unittest.main()

    suite = unittest.TestLoader().loadTestsFromTestCase(TestInterpreter)
    unittest.TextTestRunner(verbosity=2).run(suite)


# -*- coding: utf-8 -*-
import unittest
from unittest import TestCase
from video_brain.graph_parser.visualizator import Visualizator
import pickle

class TestVisualizator(TestCase):

    def test_visualize(self):

        visualizator = Visualizator()
        visualizator.visualize('/home/leo/Desktop/graph_deep_residual/water_plant_sky_snow_cloud_rock_beach_sand_vegetation/database/')

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
import unittest
from unittest import TestCase
from video_brain.graph_parser.network_evaluator import NetworkEvaluator

class TestGraphBrowser(TestCase):

    def test_fit(self):

        labels = ['water', 'plant', 'sky', 'snow', 'cloud', 'rock', 'beach', 'sand', 'vegetation']

        network_evaluator = NetworkEvaluator()
        network_evaluator.eval_network('/home/leo/Desktop/graph_deep_residual/water_plant_sky_snow_cloud_rock_beach_sand_vegetation/', labels)

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
from unittest import TestCase
from video_brain.video_parser import VideoParser

class TestBrowsing(TestCase):

    def test_graph_browsing(self):


        video_parser = VideoParser(['../../../../../rushs/AF-AFR-01-003_500k.f4v'])

        tags = video_parser.get_tags_naive()
        print(tags)




from ..graph_parser.graph_browser import GraphBrowser
from key_frames import get_key_frames
from subprocess import call
from motion_detection import motion_detection
from skvideo.io import vread
import numpy as np
import logging
import subprocess
import pickle
import os
import sys
import glob
caffe_root = '/home/leo/caffe/'  # this file should be run from {caffe_root}/examples (otherwise change this line)
sys.path.insert(0, caffe_root + 'python')
import caffe
import cv2

# sys.path.remove('/home/leo/caffe/')
lisa_root = '/home/leo/lisa-caffe/'  # this file should be run from {caffe_root}/examples (otherwise change this line)
sys.path.insert(0, lisa_root + 'python')
import caffe as lisa_caffe
reload(lisa_caffe)
reload(lisa_caffe.io)
# logging.basicConfig(filename='../logs/videolib.txt',level=logging.DEBUG)
import collections
from pygraphml import GraphMLParser
import networkx as nx

def Tree():
    return collections.defaultdict(Tree)

class VideoParser():
    """
    API enabling to call video processing services  
    """

    def __init__(self, *videopaths):
        """
        Args:
            @type videopaths: Array-like of string(s)
            @param videopaths: Absolute path(s) to the video(s) to process
        """

        self.video_paths = videopaths[0]

    def initialize_transformer(self, image_mean, is_flow):

        shape = (10 * 16, 3, 227, 227)
        transformer = lisa_caffe.io.Transformer({'data': shape})
        channel_mean = np.zeros((3, 227, 227))
        for channel_index, mean_val in enumerate(image_mean):
            channel_mean[channel_index, ...] = mean_val
        transformer.set_mean('data', channel_mean)
        transformer.set_raw_scale('data', 255)
        transformer.set_channel_swap('data', (2, 1, 0))
        transformer.set_transpose('data', (2, 0, 1))
        transformer.set_is_flow('data', is_flow)
        return transformer

    def LRCN_classify_video(self, frames, net, transformer, is_flow):
        clip_length = 16
        offset = 8
        input_images = []
        for im in frames:
            input_im = lisa_caffe.io.load_image(im)
            if (input_im.shape[0] < 240):
                input_im = lisa_caffe.io.resize_image(input_im, (240, 320))
            input_images.append(input_im)
        vid_length = len(input_images)
        input_data = []
        for i in range(0, vid_length, offset):
            if (i + clip_length) < vid_length:
                input_data.extend(input_images[i:i + clip_length])
            else:  # video may not be divisible by clip_length
                input_data.extend(input_images[-clip_length:])
        output_predictions = np.zeros((len(input_data), 8))
        for i in range(0, len(input_data), clip_length):
            clip_input = input_data[i:i + clip_length]
            clip_input = lisa_caffe.io.oversample(clip_input, [227, 227])
            clip_clip_markers = np.ones((clip_input.shape[0], 1, 1, 1))
            clip_clip_markers[0:10, :, :, :] = 0
            #    if is_flow:  #need to negate the values when mirroring
            #      clip_input[5:,:,:,0] = 1 - clip_input[5:,:,:,0]
            caffe_in = np.zeros(np.array(clip_input.shape)[[0, 3, 1, 2]], dtype=np.float32)
            for ix, inputs in enumerate(clip_input):
                caffe_in[ix] = transformer.preprocess('data', inputs)
            out = net.forward_all(data=caffe_in, clip_markers=np.array(clip_clip_markers))
            output_predictions[i:i + clip_length] = np.mean(out['probs'], 1)

        print(output_predictions)

        return np.mean(output_predictions, 0).argmax(), output_predictions

    def get_movements(self):
        """
        Process video(s) to get the motion of the camera

        Return:
            @type movements: Array-like of String
            @param movements: Motion of the camera for each video
        """

        movements = {}

        for videopath in self.video_paths:

            print(videopath)

            cap = cv2.VideoCapture(videopath)
            cap = vread(videopath, backend='libav')

            try:
                movement = motion_detection(cap)
            except:
                movement = ''
                logging.exception('motion detection : ' + str(videopath))

            movements[videopath] = movement

            # net = lisa_caffe.Net(
            #     os.path.dirname(os.path.realpath(__file__)) + '/../resources/motion/deploy_lstm.prototxt',
            #     os.path.dirname(os.path.realpath(__file__)) + '/../resources/motion/weights.caffemodel',
            #     lisa_caffe.TEST)
            #
            # ucf_mean_RGB = np.zeros((3, 1, 1))
            # ucf_mean_RGB[0, :, :] = 103.939
            # ucf_mean_RGB[1, :, :] = 116.779
            # ucf_mean_RGB[2, :, :] = 128.68
            #
            # transformer_RGB = self.initialize_transformer(ucf_mean_RGB, False)
            # call([os.path.join(os.path.dirname(__file__), '../resources/motion/extract_frames.sh'), videopath, str(30)], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            #
            # RGB_frames = glob.glob('RushApp/static/media/frames/'+videopath.split('/')[-1].split('.')[0]+'/*.jpg')
            #
            # class_RGB_LRCN, predictions_RGB_LRCN = self.LRCN_classify_video(RGB_frames, net, transformer_RGB, False)
            #
            # labels = pickle.load(open(os.path.join(os.path.dirname(__file__), '../resources/motion/labels.pkl'), 'r'))
            #
            # movement = labels[class_RGB_LRCN]
            #
            # movements[videopath] = movement

        return movements

    def get_landscapes(self):
        """
        Process video(s) to get the main theme (mountain / city / country ...) of the video(s)

        Return:
            @type themes: Array-like of String
            @param themes: main theme for each video
        """

        # clf = pickle.load(open(os.path.dirname(os.path.realpath(__file__))+'/resources/landscape/model.p', 'r'))
        net = caffe.Net(os.path.dirname(os.path.realpath(__file__))+'/../resources/themes/deploy.prototxt',
                        os.path.dirname(os.path.realpath(__file__))+'/../resources/themes/snapshot_iter_7100.caffemodel',
                        caffe.TEST)
        labels = pickle.load(open(os.path.dirname(os.path.realpath(__file__))+'/../resources/themes/labels.pkl', 'r'))
        # net, transformer = init_network('bvlc_reference_caffenet')

        all_key_frames = get_key_frames(self.video_paths)

        themes = {}
        for key_frames, videopath in zip(all_key_frames, self.video_paths):
            
            potential_landscape = []
            cpt = 1
            for key_frame in key_frames:

                img = key_frame[:, :, ::-1]

                transformer = caffe.io.Transformer({'data': net.blobs['data'].data.shape})
                transformer.set_transpose('data', (2, 0, 1))
                transformed_image = transformer.preprocess('data', img)

                transformed_image = np.reshape(transformed_image, (1, 3, 224, 224))

                net.blobs['data'].reshape(*transformed_image.shape)
                net.blobs['data'].data[...] = transformed_image

                net.forward()

                proba = np.absolute(net.blobs['proba2'].data[0, ...])

                y_pred = np.argmax(proba, 0)

                landscape = labels[y_pred]

                potential_landscape.append(landscape)

            try:
                all_landscape, landscape_counts = np.unique(potential_landscape, return_counts=True)
                themes[videopath] = all_landscape[np.argsort(landscape_counts)[::-1][0]]
            except IndexError:
                print(potential_landscape)
                themes[videopath] = ''

        return themes

    def get_tags_naive(self):

        all_key_frames = get_key_frames(self.video_paths)

        weights_files = glob.glob(os.path.join(os.path.dirname(__file__), '../resources/tags/weights/*.caffemodel'))

        # concepts_graph = nx.read_graphml(os.path.join(os.path.dirname(__file__), '../resources/onto.graphml'))

        tags = {}
        tags_probas = Tree()
        for key_frames, videopath in zip(all_key_frames, self.video_paths):

            for weights_file in weights_files:

                str_labels = weights_file.split('.')[-2].split('/')[-1]

                if 'man_work' in str_labels:
                    str_labels = str_labels.replace('man_work', 'man-work')

                threshold_probas = pickle.load(
                    open(os.path.dirname(os.path.realpath(__file__)) + '/../resources/tags/thresholds/' + str_labels + '.pkl',
                         'r'))

                dst = os.path.join(os.path.dirname(__file__), '../resources/tags/deploy')
                os.rename(dst + '/' + str_labels + '.prototxt', dst + '/old_' + str_labels + '.prototxt')
                new_deploy = open(dst + '/' + str_labels + '.prototxt', 'w')
                score_layer = False
                with open(dst + '/old_'+str_labels+'.prototxt', 'r') as deploy:
                    for line in deploy:
                        if 'name: "score"' in line:
                            score_layer = True
                        if 'num_output:' in line and score_layer is True:
                            new_deploy.write('     num_output: ' + str(len(threshold_probas)) + '\n')
                        else:
                            new_deploy.write(line)

                new_deploy.close()
                os.remove(dst + '/old_' + str_labels + '.prototxt')

                net = caffe.Net(os.path.join(os.path.dirname(__file__), '../resources/tags/deploy/' + str_labels + '.prototxt'),
                                weights_file,
                                caffe.TEST)

                probas = []
                for key_frame in key_frames:

                    img = key_frame[:, :, ::-1]

                    transformer = caffe.io.Transformer({'data': net.blobs['data'].data.shape})
                    transformer.set_transpose('data', (2, 0, 1))
                    transformed_image = transformer.preprocess('data', img)

                    transformed_image = np.reshape(transformed_image, (1, 3, 227, 227))

                    net.blobs['data'].reshape(*transformed_image.shape)
                    net.blobs['data'].data[...] = transformed_image

                    net.forward()

                    proba = np.absolute(net.blobs['proba'].data[0, ...])
                    probas.append(proba)

                probas = np.array(probas)
                probas = np.mean(probas, 0)

                labels = np.array(str_labels.split('_'))

                if 'nature' in str_labels:
                    estlist = probas > [0.11, 0.20, 0.5, 0.5]
                elif 'building' in str_labels:
                    estlist = probas > [0.5, 0.5, 0.51, 0.5, 0.5]
                elif 'sand' in str_labels:
                    estlist = probas > [0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.5]

                if 1 in estlist:
                    est_classes = labels[np.where(estlist)[0]]
                else:
                    est_classes = []

                print(labels)
                print(probas)
                print('\n')

                for label, proba in zip(labels, probas):
                    tags_probas[label] = proba

                if videopath in tags.keys():
                    tags[videopath] = tags[videopath] + list(est_classes)
                else:
                    tags[videopath] = list(est_classes)

        return tags

    def get_tags(self):

        all_key_frames = get_key_frames(self.video_paths)

        graph_parser = GraphBrowser()

        str_labels = ['man_work_nature_animal_person', 'water_plant_sky_snow_cloud_rock_beach_sand_vegetation']

        tags = {}
        for key_frames, videopath in zip(all_key_frames, self.video_paths):

            labels_probas = {}
            # labels_probas[str_labels[0]] = []
            # labels_probas[str_labels[1]] = []
            for key_frame in key_frames:

                predicted_tags = graph_parser.predict(key_frame)

                for predicted_tag in predicted_tags:

                    if predicted_tag[1] is not None:

                        str_labels = '_'.join(predicted_tag[0])
                        if str_labels in labels_probas.keys():
                            labels_probas[str_labels].append((predicted_tag[0], predicted_tag[1]))
                        else:
                            labels_probas[str_labels] = []
                            labels_probas[str_labels].append((predicted_tag[0], predicted_tag[1]))

            for str_labels, all_probas in labels_probas.iteritems():

                labels = np.array(all_probas[0][0])

                probas = []
                for proba in all_probas:
                    probas.append(proba[1])

                probas = np.array(probas)
                probas = np.mean(probas, 0)

                threshold_probas = pickle.load(open(os.path.dirname(os.path.realpath(__file__)) + '/../resources/tags/'+str_labels+'/thresholds.pkl', 'r'))

                estlist = np.array(probas > threshold_probas)

                if 1 in estlist:
                    ids = np.where(estlist)[0]
                    est_classes = labels[ids]
                else:
                    est_classes = []

                if videopath in tags.keys():
                    tags[videopath] = tags[videopath] + list(est_classes)
                else:
                    tags[videopath] = list(est_classes)

        return tags

    #     classes = np.array(['person', 'animal', 'nature', 'eau', 'ouvrage'])
    #
        # net = caffe.Net(os.path.dirname(os.path.realpath(__file__))+'/resources/tags_cnn/testnet.prototxt',
        #                 os.path.dirname(os.path.realpath(__file__))+'/resources/tags_cnn/network.caffemodel',
        #                 caffe.TEST)
    #
    #     all_key_frames = get_key_frames(self.video_paths)
    #
    #
    #     tags = {}
    #     for key_frames, videopath in zip(all_key_frames, self.video_paths):
    #
    #         probas = []
    #         for key_frame in key_frames:
    #
                # img = key_frame[:,:,::-1]
                #
                # transformer = caffe.io.Transformer({'data': net.blobs['data'].data.shape})
                # transformer.set_transpose('data', (2,0,1))
                # transformed_image = transformer.preprocess('data', img)
                #
                # transformed_image = np.reshape(transformed_image, (1,3,227,227))
                #
                # net.blobs['data'].reshape(*transformed_image.shape)
                # net.blobs['data'].data[...] = transformed_image
                #
                # net.forward()
                #
                # proba = np.absolute(net.blobs['proba3'].data[0, ...])
                # probas.append(proba)

            # probas = np.array(probas)
            # probas = np.mean(probas, 0)
            #
            # estlist0 = (probas[0] > threshold_probas[0]).astype(int)
            # estlist1 = (probas[1] > threshold_probas[1]).astype(int)
            # estlist2 = (probas[2] > threshold_probas[2]).astype(int)
            # estlist3 = (probas[3] > threshold_probas[3]).astype(int)
            # estlist4 = (probas[4] > threshold_probas[4]).astype(int)
            #
            # estlist = np.concatenate((estlist0, estlist1, estlist2, estlist3, estlist4), 0)
            #
            # estlist = probas > threshold_probas
            #
            # if 1 in estlist:
            #     est_classes = classes[np.where(estlist)[0]]
            # else:
            #     est_classes = []
            #
            # tags[videopath] = list(est_classes)
    #
    #     return tags


        # img = cv2.imread('/home/leo/Projets/castor/images/rushs/val/PA-AUS-16-014_1.png')
     #    img = img[:,:,::-1]

     #    print(probas)
     #    # print(probas.shape)

     #     return est_classes


# vlib = Videolib(['/home/leo/Projets/castor/rushs/AS-IND-06-544_500k.f4v'])
# tags = vlib.get_tags()
# print(tags)



from mpl_toolkits.axes_grid1 import host_subplot
from sklearn.metrics import roc_curve, precision_score, recall_score
from matplotlib.backends.backend_pdf import FigureCanvasPdf as FigureCanvas
import matplotlib.pyplot as plt
import numpy as np
import threading
import pickle
import re
import glob
import os
import sys
import settings
import lmdb

sys.path.insert(0, settings.caffe_root + 'python')
import caffe

PATIENCE = 5

class NetworkEvaluator():

    def __init__(self):

        self.ps = []

    def draw_roc_curve(self, gtlist, probas, iter, best_output, working_directory, classes):

        chosen_thresholds = []
        for key, label in enumerate(classes):

            label_gtlist = gtlist[:, key, ]
            label_probas = probas[:, key]

            fpr, tpr, thresholds = roc_curve(label_gtlist, label_probas)

            best_id = np.min(np.where(tpr > 0.75)[0].astype(int))

            best_threshold = thresholds[best_id]
            chosen_fpr = fpr[best_id]
            chosen_tpr = tpr[best_id]
            chosen_thresholds.append(best_threshold)

            fig = plt.figure()
            canvas = FigureCanvas(fig)
            ax = fig.add_subplot(1, 1, 1)
            ax.plot(fpr, tpr)
            plt.title('ROC curve for ' + str(label))
            plt.xlabel('False Positive Rate')
            plt.ylabel('True Positive Rate')
            fig.savefig(working_directory + 'eval/roc_' + label + '.jpg')

            estlist = (label_probas > best_threshold).astype(int)
            precision = precision_score(label_gtlist, estlist)
            recall = recall_score(label_gtlist, estlist)

            with open(working_directory + 'eval/report.txt', "ab") as report:
                report.write(label + ' : \r\n')
                report.write('Probability threshold chosen : ' + str(best_threshold) + '\r\n')
                report.write('FPR : ' + str(chosen_fpr) + '\r\n')
                report.write('TPR : ' + str(chosen_tpr) + '\r\n')
                report.write('Precision : ' + str(precision) + '\r\n')
                report.write('Recall : ' + str(recall) + '\r\n')
                report.write('\r\n')
            report.close()
        report = open(working_directory + 'eval/report.txt', "ab")
        report.write('Best iter : ' + str(iter) + '\n')
        if best_output != None:
            report.write('Best Output : ' + str(best_output) + '\n')
        report.close()
        pickle.dump(chosen_thresholds, open(working_directory + 'eval/thresholds.pkl', 'w'))

    def learning_curves(self, working_directory):

        f = open(working_directory + '/output.txt', 'r')

        training_iterations = []
        training_loss = []

        test_iterations = []
        test_accuracy = []
        test_loss = []

        # model = working_directory.split('/')[1]
        model = 'deep_residual'

        if model == 'googlenet':
            test_loss2 = []
            test_loss3 = []
            cpt = 1

        for line in f:

            if 'Test net output #0' in line:
                loss = float(line.strip().split(' = ')[-1].split(' ')[0])
                test_loss.append(loss)

            elif 'Test net output #1' in line:
                loss = float(line.strip().split(' = ')[-1].split(' ')[0])
                test_loss2.append(loss)

            elif 'Test net output #2' in line:
                loss = float(line.strip().split(' = ')[-1].split(' ')[0])
                test_loss3.append(loss)

            if 'Train net output' in line:
                loss = float(line.strip().split(' = ')[-1].split(' ')[0])
                training_loss.append(loss)

            if '] Iteration ' in line and 'Testing net (#0)' in line:
                arr = re.findall(r'ion \b\d+\b,', line)
                test_iterations.append(int(arr[0].strip(',')[4:]))

            if '] Iteration ' in line and 'loss = ' in line:
                arr = re.findall(r'ion \b\d+\b,', line)
                training_iterations.append(int(arr[0].strip(',')[4:]))

        print 'train iterations len: ', len(training_iterations)
        print 'train loss len: ', len(training_loss)
        print 'test loss len: ', len(test_loss)
        print 'test iterations len: ', len(test_iterations)
        print 'test accuracy len: ', len(test_accuracy)

        f.close()

        host = host_subplot(111)  # , axes_class=AA.Axes)
        plt.subplots_adjust(right=0.75)

        par1 = host.twinx()

        host.set_xlabel("iterations")
        host.set_ylabel("loss")
        # par1.set_ylabel("validation accuracy")

        p1, = host.plot(training_iterations, training_loss, label="training loss")


        if model == 'alexnet':
            p2, = host.plot(test_iterations, test_loss, label="validation loss")
            # p3, = host.plot(test_iterations, test_accuracy, label="validation accuracy")
            best_test_loss = test_loss
            best_output = None

        elif model == 'deep_residual':
            p2, = host.plot(test_iterations, test_loss, label="validation loss")
            best_test_loss = test_loss
            best_output = None

        elif model == 'googlenet':

            min1 = min(test_loss)
            min2 = min(test_loss2)
            min3 = min(test_loss3)

            if min1 < min2 and min1 < min3:
                best_test_loss = test_loss
                best_output = 1
            elif min2 < min1 and min2 < min3:
                best_test_loss = test_loss2
                best_output = 2
            elif min3 < min1 and min3 < min1:
                best_test_loss = test_loss3
                best_output = 3
            elif min2 == min3:
                best_test_loss = test_loss2
                best_output = 2
            elif min1 == min2:
                best_test_loss = test_loss2
                best_output = 2
            elif min1 == min3:
                best_test_loss = test_loss3
                best_output = 3

            p2, = host.plot(test_iterations, best_test_loss, label="validation loss")
            # p3, = host.plot(test_iterations, test_loss2, label="validation loss 2")
            # p4, = host.plot(test_iterations, test_loss3, label="validation loss 3")

        host.legend(loc=0)

        host.axis["left"].label.set_color(p1.get_color())
        # par1.axis["right"].label.set_color(p2.get_color())

        plt.draw()
        plt.savefig(working_directory + '/eval/learning_curves.jpg')

        with open(working_directory + 'network/solver.prototxt', 'r') as solver_file:

            for line in solver_file:
                if 'test_interval' in line:
                    interval = int(line.split(' ')[-1])

        return (np.argmin(best_test_loss) + 1) * interval, best_output

    def eval_network(self, working_directory, labels):

        # self.set_directory_structure(working_directory + '/output.txt')
        if not os.path.exists(working_directory + '/eval'):
            os.makedirs(working_directory + '/eval')

        best_iter, best_output = self.learning_curves(working_directory)

        net = caffe.Net(working_directory + 'network/valnet.prototxt',
                        working_directory + 'training/snapshot_iter_' + str(best_iter) + '.caffemodel',
                        caffe.TEST)

        net.forward()

        gtlist = net.blobs['label'].data.astype(np.int)
        gtlist = np.squeeze(gtlist)

        if best_output != None:
            probas = np.absolute(net.blobs['proba' + str(best_output)].data)
        else:
            probas = np.absolute(net.blobs['proba'].data)

        self.draw_roc_curve(gtlist, probas, best_iter, best_output, working_directory, labels)

    def evaluate_and_get_thresholds(self, working_directory, classes):

        net = caffe.Net(working_directory + '/deploy.prototxt',
                        working_directory + '/weights.caffemodel',
                        caffe.TEST)

        transformer = caffe.io.Transformer({'data': net.blobs['data'].data.shape})
        transformer.set_transpose('data', (2, 0, 1))

        val_images_env = lmdb.open(os.path.dirname(working_directory) + '/val_rush_images', readonly=True)
        val_labels_env = lmdb.open(os.path.dirname(working_directory) + '/val_rush_labels', readonly=True)

        gtlist = []
        probas = []
        with val_images_env.begin() as txn_val_images:
            with val_labels_env.begin() as txn_val_labels:
                images_cursor = txn_val_images.cursor()
                targets_cursor = txn_val_labels.cursor()

                length = txn_val_images.stat()['entries']
                print(length)

                for key, ((key_images, value_images), (key_targets, value_targets)) in enumerate(zip(images_cursor, targets_cursor)):

                    if key == 10000:
                        break

                    datum = caffe.proto.caffe_pb2.Datum()
                    datum.ParseFromString(value_images)
                    flat_img = np.fromstring(datum.data, dtype=np.uint8)
                    img = flat_img.reshape(datum.channels, datum.height, datum.width)
                    img = img.transpose((1, 2, 0))

                    datum = caffe.proto.caffe_pb2.Datum()
                    datum.ParseFromString(value_targets)
                    target = caffe.io.datum_to_array(datum)

                    gtlist.append(target[0][0])

                    transformed_image = transformer.preprocess('data', img)
                    transformed_image = np.reshape(transformed_image, (1, 3, 227, 227))
                    net.blobs['data'].reshape(*transformed_image.shape)
                    net.blobs['data'].data[...] = transformed_image

                    net.forward()

                    proba = np.absolute(net.blobs['proba'].data[0, ...])
                    probas.append(proba)

        gtlist = np.array(gtlist)
        probas = np.array(probas)

        chosen_thresholds = []
        for key, label in enumerate(classes):

            label_gtlist = gtlist[:, key, ]
            label_probas = probas[:, key]

            fpr, tpr, thresholds = roc_curve(label_gtlist, label_probas)

            best_id = np.min(np.where(tpr > 0.7)[0].astype(int))

            best_threshold = thresholds[best_id]
            chosen_fpr = fpr[best_id]
            chosen_tpr = tpr[best_id]
            chosen_thresholds.append(best_threshold)

            # if not os.path.exists('eval'):
            #     os.makedirs('eval')

            fig = plt.figure()
            canvas = FigureCanvas(fig)
            ax = fig.add_subplot(1, 1, 1)
            ax.plot(fpr, tpr)
            plt.title('ROC curve for ' + str(label))
            plt.xlabel('False Positive Rate')
            plt.ylabel('True Positive Rate')
            fig.savefig(working_directory + '/eval/roc_' + label + '.jpg')

            estlist = (label_probas > best_threshold).astype(int)
            precision = precision_score(label_gtlist, estlist)
            recall = recall_score(label_gtlist, estlist)

            if not os.path.exists(working_directory + '/eval/'):
                os.makedirs(working_directory + '/eval/')

            with open(working_directory + '/eval/report.txt', "ab") as report:
                report.write(label + ' : \r\n')
                report.write('Probability threshold chosen : ' + str(best_threshold) + '\r\n')
                report.write('FPR : ' + str(chosen_fpr) + '\r\n')
                report.write('TPR : ' + str(chosen_tpr) + '\r\n')
                report.write('Precision : ' + str(precision) + '\r\n')
                report.write('Recall : ' + str(recall) + '\r\n')
                report.write('\r\n')
            report.close()

        pickle.dump(chosen_thresholds, open(working_directory + '/eval/thresholds.pkl', 'w'))

    def monitor(self):

        threading.Timer(10.0, self.monitor).start()

        new_ps = [p for p in self.ps if p[0].poll() == None]
        self.ps = new_ps

        for key, p in enumerate(self.ps):

            labels = p[2]
            working_directory = p[1]
            process = p[0]

            old_loss = 100
            cpt = 0
            best_loss = 100
            best_iter = None
            with open(working_directory + "/output.txt", "r") as output_file:
                for line in output_file:

                    if '] Iteration ' in line and 'Testing net (#0)' in line:
                        arr = re.findall(r'ion \b\d+\b,', line)
                        iter = int(arr[0].strip(',')[4:])

                    if 'Test net output #0:' in line:
                        new_loss = float(line.strip().split(' = ')[-1].split(' ')[0])

                        if new_loss < best_loss:
                            best_loss = new_loss
                            best_iter = iter
                            cpt = 0
                        else:
                            cpt += 1

                        if cpt >= PATIENCE:
                            process.kill()

                            models_file = glob.glob(working_directory + '/training/*')

                            for model_file in models_file:

                                if not str(best_iter) in model_file:
                                    os.remove(model_file)
                            self.eval_network(working_directory, labels)
                            break
                        old_loss = new_loss

            # def cmp_iteration(namefile1, namefile2):
            #
            #     iter1 = int(namefile1.split('/')[-1].split('.')[0].split('_')[-1])
            #     iter2 = int(namefile2.split('/')[-1].split('.')[0].split('_')[-1])
            #
            #     return 1 if iter1 > iter2 else -1
            #
            # weights_files = glob.glob(working_directory + 'training/*.caffemodel')
            # weights_files.sort(cmp_iteration)
            #
            # while len(weights_files) > (PATIENCE + 1):
            #     os.remove(weights_files[0])
            #     os.remove(weights_files[0].replace('caffemodel', 'solverstate'))
            #     weights_files.pop(0)

    def add_training(self, p):
        self.ps.append(p)
